unit SerCom;

interface

uses
  Windows;

const
  SER_OK = 0;      // Kein Fehler
  SER_NIL = -1;    // SerialPtr nicht initialisiert / Zuwenig RAM fr Puffer
  SER_ERR = -2;    // Unzulssige Parameter oder Schnittstelle geschlossen
  SER_SYS = -3;    // Fehlermeldung vom Betriebssystem. Nummer in "SerialCode"

 type
  SerialRec = record
    SysErr: longint;
    ComName: array [0..5] of char;
    hFile: tHandle;
    sBaud: longint;
    sData: integer;
    sStop: integer;
    sParity: char;
    Output: longint;
    SizeIn: longint;
    SizeOut: longint;
    TimeOut: tCommTimeOuts;
  end;

  SerialPtr = ^SerialRec;
  SerialErr = longint;

{-------------------------------------------------------}
{  Initialisierung / Deinitialisierung                  }
{-------------------------------------------------------}
function SerialInit(PortNr, InBuffer, OutBuffer: integer): SerialPtr;
function SerialCode(SP: SerialPtr): SerialErr;
function SerialExit(SP: SerialPtr): SerialErr;

{-------------------------------------------------------}
{  ffnen / Schlieen / Parametrieren                   }
{-------------------------------------------------------}
function SerialPara(SP: SerialPtr; Baud: longint; Data, Stop: integer; Parity: char): SerialErr;
function SerialOpen(SP: SerialPtr): SerialErr;
function SerialStop(SP: SerialPtr): SerialErr;

{-------------------------------------------------------}
{  Senden / Empfangen                                   }
{-------------------------------------------------------}
function SerialXmit(SP: SerialPtr; data: char): SerialErr;
function SerialRecv(SP: SerialPtr; var data: char): SerialErr;
function SerialDone(SP: SerialPtr): SerialErr;
procedure SerialXmitstring(SP: SerialPtr; data:string);

function SerialRSet(SP: SerialPtr; Value: byte): SerialErr;
// function SerialRGet(SP: SerialPtr): SerialErr;


implementation

function SetParameter(SP: SerialPtr): SerialErr;
var dcb: tdcb;
begin
  result := SER_OK;
  if (SP <> nil) then with SP^ do
    begin
      if hFile <> INVALID_HANDLE_VALUE then
      begin
        if GetCommState(hfile, dcb) then with dcb do
          begin
            BaudRate := sBaud;
            ByteSize := sData;
            case sStop of
              1, 10: StopBits := 0;
              3, 15: StopBits := 1;
              2, 20: StopBits := 2;
            end;
            case sParity of
              'N', 'n': Parity := 0;
              'O', 'o': Parity := 1;
              'E', 'e': Parity := 2;
              'M', 'm': Parity := 3;
              'S', 's': Parity := 4;
            end;
            if not SetCommState(hfile, dcb) then
            begin
              SysErr := getLastError;
              result := SER_SYS;
            end;
          end
        else begin
          SysErr := getLastError;
          result := SER_SYS;
        end;
      end;
    end
  else result := SER_NIL;
end;


{ private }
{----------------------------------------------------------------------}
{ public }


{----------------------------------------------------------------------}
{  Funktion "SerialInit"                                               }
{  initialisiert die Verwendung der seriellen Schnittstelle. Diese     }
{  Funktion wird blicherweise einmal zu Beginn des Programms aufge-   }
{  rufen.                                                              }
{  Der Parameter PortNr bezeichnet die zu verwendende Schnittstelle.   }
{  Die Parameter InBuffer und OutBuffer geben die Gre der Zwischen-  }
{  puffer fr das asynchrone Senden und Empfangen an.                  }
{  Eine typische Gre von InBuffer und OutBuffer ist 1024.            }
{  Das Ergebnis der Funktion ist ein Handle fr die zu verwendende     }
{  Schnittstelle. In allen weiteren Routinen mu dieses Handle         }
{  angegeben werden. Durch die Identifizierung ber ein Handle ist es  }
{  mglich mehrere Schnittstellen gleichzeitig zu ffnen. Das Gegen-   }
{  stck zu "SerialInit" ist die Funktion "SerialExit".                }
{----------------------------------------------------------------------}

function SerialInit(PortNr, InBuffer, OutBuffer: integer): SerialPtr;
var SP: SerialPtr;
begin
  New(SP);
  if (SP <> nil) then with SP^ do
    begin
      ComName[0] := 'C';
      ComName[1] := 'O';
      ComName[2] := 'M';
      ComName[3] := char(PortNr + 48);
      ComName[4] := ':';
      ComName[5] := #0;
      hFile := INVALID_HANDLE_VALUE;
      sBaud := 9600;
      sData := 8;
      sStop := 10;
      sParity := 'N';
      Fillchar(TimeOut, sizeof(TimeOut), 0);
      with TimeOut do
      begin
        ReadIntervalTimeout := MAXDWORD;
        ReadTotalTimeoutConstant := 0;
        ReadTotalTimeoutMultiplier := 0;
      end;
      SizeIn := InBuffer;
      SizeOut := OutBuffer;
    end;
  SerialInit := SP;
end;


{----------------------------------------------------------------------}
{  Funktion "SerialCode"                                               }
{  gibt eine erweiterte Fehlermeldung zurck wenn eine andere Routine  }
{  zuvor das Ergebnis "SER_SYS" (-3) geliefert hat. Die Fehlernummer   }
{  ist die Io-Meldung des Betriebssystems.                             }
{----------------------------------------------------------------------}

function SerialCode(SP: SerialPtr): SerialErr;
begin
  if (SP <> nil) then with SP^ do
    begin
      result := SysErr;
      SysErr := 0;
    end
  else result := SER_NIL;
end;


{----------------------------------------------------------------------}
{  Funktion "SerialExit"                                               }
{  ist das Gegenstck zu "SerialInit". Diese Funktion gibt den beleg-  }
{  ten Speicher fr das Handle zu einer Schnittstelle wieder frei.     }
{  Ist die Schnittstelle zum Zeitpunkt des Aufrufes noch geffnet, so  }
{  wird sie zunchst geschlossen.                                      }
{----------------------------------------------------------------------}

function SerialExit(SP: SerialPtr): SerialErr;
begin
  if (SP <> nil) then
  begin
    result := SerialStop(SP);
    Dispose(SP);
  end
  else result := SER_NIL;
end;


{----------------------------------------------------------------------}
{  Funktion "SerialPara"                                               }
{  parametriert die serielle Schnittstelle. Es ist dabei unerheblich,  }
{  ob die Schnittstelle bereits geffnet ist oder nicht.               }
{  Fr eine ordentliche Kommunikation ist es erforderlich, da alle    }
{  Parameter mit denen der Gegenstelle bereinstimmen. Viele Modems    }
{  knnen die verwendeten Parameter selbstndig erkennen und knnen    }
{  daher mit verschiedenen Parametern angesprochen werden.             }
{  Die Standardeinstellung fr Daten- und Stopbits sowie Paritt sind  }
{  8 - 1 - Keine (None).                                               }
{----------------------------------------------------------------------}

function SerialPara(SP: SerialPtr; Baud: longint; Data, Stop: integer; Parity: char): SerialErr;
begin
  if (SP <> nil) then with SP^ do
    begin
      sBaud := Baud;
      sData := Data;
      sStop := Stop;
      sParity := Parity;
      result := SetParameter(SP);
    end
  else result := SER_NIL;
end;


{----------------------------------------------------------------------}
{  Funktion "SerialOpen"                                               }
{  ffnet die serielle Schnittstelle und ermglicht somit den Daten-   }
{  verkehr. Nachdem die Schnittstelle geffnet ist, werden alle ein-   }
{  gehenden Zeichen im Zwischenpuffer gespeichert bis diese mit der    }
{  Funktion "SerialRecv" aus dem Puffer entnommen werden.              }
{  Das Gegenstck zu "SerialOpen" ist die Funktion "SerialStop".       }
{  Die Zwischenpuffer werden beim ffnen und schlieen einer Schnitt-  }
{  stelle gelscht.                                                    }
{----------------------------------------------------------------------}

function SerialOpen(SP: SerialPtr): SerialErr;
var commprop: tCommProp;
begin
  result := SER_OK;
  if (SP <> nil) then with SP^ do
    begin
      if hFile = INVALID_HANDLE_VALUE then
      begin
        hFile := CreateFile(ComName, GENERIC_READ + GENERIC_WRITE, 0, nil, OPEN_EXISTING, 0, 0);
        if (hFile <> INVALID_HANDLE_VALUE) then
        begin
          if GetCommProperties(hfile, commprop) then
          begin
            SetupComm(hfile, SizeIn, SizeOut);
            SetParameter(SP);
            SetCommTimeouts(hfile, TimeOut);
            SetCommMask(hfile, 0);
            Output := commprop.dwCurrentTxQueue;
          end
          else begin
            SysErr := GetLastError;
            result := SER_SYS;
          end;
        end
        else begin
          SysErr := GetLastError;
          result := SER_SYS;
        end;
      end;
    end
  else result := SER_NIL;
end;


{----------------------------------------------------------------------}
{  Funktion "SerialStop"                                               }
{  schliet die serielle Schnittstelle. Die Datenkommunikation wird    }
{  unterbunden und der Gegenstelle wird auf den Handshakeleitungen     }
{  signalisiert, da die Schnittstelle geschlossen ist. Ein ordent-    }
{  lich konfiguriertes Modem sollte dadurch eine bestehende Telefon-   }
{  verbindung trennen.                                                 }
{----------------------------------------------------------------------}

function SerialStop(SP: SerialPtr): SerialErr;
begin
  if (SP <> nil) then with SP^ do
    begin
      if hFile <> INVALID_HANDLE_VALUE then
      begin
        EscapeCommFunction(hFile, CLRRTS);
        SetCommMask(hfile, 0);
        CloseHandle(hFile);
        hFile := INVALID_HANDLE_VALUE;
      end;
      result := SER_OK;
    end
  else result := SER_NIL;
end;


{----------------------------------------------------------------------}
{  Funktion "SerialXmit"                                               }
{  bergibt ein Zeichen an den Sendepuffer. Falls die Schnittstelle    }
{  zu diesem Zeitpunkt geffnet ist, werden die Zeichen im Hinter-     }
{  grund asynchron bertragen. Die Funktion kehrt nach ihrem Aufruf    }
{  schnell zurck und wartet nicht auf das Ende der bertragung.       }
{  Zeichenketten und binre Arrays werden vom Aufrufer in einer        }
{  Schleife an diese Funktion bergeben.                               }
{  Mit der Funktion "SerialDone" kann geprft werden, ob schon alle    }
{  Zeichen bertragen wurden.                                          }
{----------------------------------------------------------------------}

function SerialXmit(SP: SerialPtr; data: char): SerialErr;
var
  count: DWORD;
  comstat: tComstat;
begin
  if (SP <> nil) then with SP^ do
    begin
      if hFile <> INVALID_HANDLE_VALUE then
      begin
        ClearCommError(hfile, count, @comstat);
        if (((Output - SerialErr(comstat.cbOutQue)) - 1) <= 0) then
        begin
          if WriteFile(hfile, data, 1, count, nil) then result := count
          else begin
            SysErr := GetLastError;
            result := SER_SYS;
          end;
        end
        else result := SER_NIL;
      end
      else result := SER_ERR;
    end
  else result := SER_NIL;
end;

procedure SerialXmitstring(SP: SerialPtr; data:string);
var t: SerialErr;
    i: byte;
begin
     for i:=1 to length(data)
     do
       t:=SerialXmit(SP,data[i]);
end;


{----------------------------------------------------------------------}
{  Funktion "SerialRecv"                                               }
{  holt das nchste Zeichen aus dem Empfangspuffer. Voraussetzung ist  }
{  eine geffnete Schnittstelle. Ist das Ergebnis dieser Funktion "1"  }
{  so wurde die Funktion erfolgreich ausgefhrt. Alle anderen Werte    }
{  bedeuten, da kein Zeichen empfangen wurde.                         }
{----------------------------------------------------------------------}

function SerialRecv(SP: SerialPtr; var data: char): SerialErr;
var count: DWORD;
begin
  if (SP <> nil) then with SP^ do
    begin
      if hFile <> INVALID_HANDLE_VALUE then
      begin
        if windows.ReadFile(hfile, data, 1, count, nil) then result := count
        else begin
          SysErr := GetLastError;
          result := SER_SYS;
        end;
      end
      else result := SER_ERR;
    end
  else result := SER_NIL;
end;


{----------------------------------------------------------------------}
{  Funktion "SerialDone"                                               }
{  ermittelt wieviele Zeichen sich momentan im Sendepuffer befinden.   }
{  Ist das Ergebnis "0", sind keine Zeichen mehr im Sendepuffer.       }
{----------------------------------------------------------------------}

function SerialDone(SP: SerialPtr): SerialErr;
begin
  if (SP <> nil)
    then result := SP^.Output
  else result := SER_NIL;
end;


{----------------------------------------------------------------------}
{  Funktion "SerialRSet"                                               }
{  setzt einzelne Handshakeleitungen der seriellen Schnittstelle.      }
{  Diese Funktion ist nur in dieser Wondows Version vorhanden.         }
{----------------------------------------------------------------------}

function SerialRSet(SP: SerialPtr; Value: byte): SerialErr;
begin
  if (SP <> nil) then with SP^ do
    begin
      if hFile <> INVALID_HANDLE_VALUE then
      begin
        EscapeCommFunction(hFile, Value);
        result := SER_OK;
      end
      else result := SER_ERR;
    end
  else result := SER_NIL;
end;

(*
{----------------------------------------------------------------------}
{  Funktion "SerialRGet"                                               }
{  liest den Status der Handshakeleitungen.                            }
{  Diese Funktion ist nur in dieser Wondows Version vorhanden.         }
{----------------------------------------------------------------------}

function SerialRGet(SP: SerialPtr): SerialErr;
var Reg: Cardinal;
begin
  if (SP <> nil) then with SP^ do
    begin
      if (hFile <> INVALID_HANDLE_VALUE) then
      begin
        if GetCommModemStatus(hFile, Reg)
          then Result := Reg
        else Result := SER_ERR;
      end
      else result := SER_ERR;
    end
  else result := SER_NIL;
end;
*)
end.


